@@ -2,22 +2,21 @@
 #
 # errorlod.R
 #
-# copyright (c) 2001-8, Karl W Broman
-# last modified Jun, 2008
+# copyright (c) 2001-9, Karl W Broman
+# last modified Apr, 2009
 # first written Apr, 2001
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 # 
 # Part of the R/qtl package
 # Contains: calc.errorlod, plot.errorlod, top.errorlod
@@ -65,10 +64,12 @@ function(cross, error.prob=0.01,
     if(type=="bc" || type=="risib" || type=="riself" || type=="dh")
       cfunc <- "calc_errorlod_bc"
     else if(type=="f2") {
-      if(chr.type=="A") cfunc <- "calc_errorlod_f2"
+      if(chr.type!="X") cfunc <- "calc_errorlod_f2"
       else cfunc <- "calc_errorlod_bc"
     }
     else if(type=="4way") cfunc <- "calc_errorlod_4way"
+    else if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib")
+      cfunc <- paste("calc_errorlod_", type, sep="")
     else 
       stop("calc.errorlod not available for cross type ", type, ".")
 
